#include "myutils.h"
#include <float.h>	// for FLT_MAX
#include "mx.h"
#include "alnparams.h"

void SetBLOSUM62();
void SetNucSubstMx(double Match, double Mismatch);
void ReadSubstMx(const string &FileName, Mx<float> &Mxf);

extern Mx<float> g_SubstMxf;
extern float **g_SubstMx;

void AlnParams::Clear()
	{
	SubstMxName = 0;
	LocalOpen = OBVIOUSLY_WRONG_PENALTY;
	LocalExt = OBVIOUSLY_WRONG_PENALTY;
	OpenA = OBVIOUSLY_WRONG_PENALTY;
	OpenB = OBVIOUSLY_WRONG_PENALTY;
	ExtA = OBVIOUSLY_WRONG_PENALTY;
	ExtB = OBVIOUSLY_WRONG_PENALTY;
	LOpenA = OBVIOUSLY_WRONG_PENALTY;
	LOpenB = OBVIOUSLY_WRONG_PENALTY;
	ROpenA = OBVIOUSLY_WRONG_PENALTY;
	ROpenB = OBVIOUSLY_WRONG_PENALTY;
	LExtA = OBVIOUSLY_WRONG_PENALTY;
	LExtB = OBVIOUSLY_WRONG_PENALTY;
	RExtA = OBVIOUSLY_WRONG_PENALTY;
	RExtB = OBVIOUSLY_WRONG_PENALTY;
	Nucleo = false;
	NucleoSet = false;
	}

bool AlnParams::Is2() const
	{
	float g = OpenA;
	float e = ExtA;
	if (OpenB != g || LOpenA != g || LOpenB != g || ROpenA != g || ROpenB != g)
		return false;
	if (ExtB != e || LExtA != e || LExtB != e || RExtA != e || RExtB != e)
		return false;
	return true;
	}

bool AlnParams::Is4() const
	{
	float g = OpenA;
	float tg = LOpenA;
	float e = ExtA;
	float te = LExtA;
	if (OpenB != g || LOpenA != tg || LOpenB != tg || ROpenA != tg || ROpenB != tg)
		return false;
	if (ExtB != e || LExtA != te || LExtB != te || RExtA != te || RExtB != te)
		return false;
	return true;
	}

const char *AlnParams::GetType() const
	{
	if (Is2())
		return "2";
	else if (Is4())
		return "4";
	return "12";
	}

void AlnParams::Init2(const float * const *Mx, float Open, float Ext)
	{
	SubstMx = Mx;
	OpenA = OpenB = LOpenA = LOpenB = ROpenA = ROpenB = Open;
	ExtA = ExtB = LExtA = LExtB = RExtA = RExtB = Ext;
	}

void AlnParams::SetLocal(float Open, float Ext)
	{
	LocalOpen = Open;
	LocalExt = Ext;
	}

void AlnParams::Init4(const float * const *Mx, float Open, float Ext,
  float TermOpen, float TermExt)
	{
	SubstMx = Mx;
	OpenA = OpenB = Open;
	LOpenA = LOpenB = ROpenA = ROpenB = TermOpen;
	ExtA = ExtB = Ext;
	LExtA = LExtB = RExtA = RExtB = TermExt;
	}

void AlnParams::LogMe() const
	{
	Log("AlnParams(%s)", GetType());
	if (Is2())
		Log(" g=%.1f e=%.1f", -OpenA, -ExtA);
	else if (Is4())
		Log(" g=%.1f tg=%.1f e=%.1f te=%.1f", -OpenA, -ExtA, -LOpenA, -LExtA);
	else
		Log(
" gA=%.1f gB=%.1f gAL=%.1f gBL=%.1f gAR=%.1f gBR=%.1f eA=%.1f eB=%.1f eAL=%.1f eBL=%.1f eAR=%.1f eBR=%.1f",
		  OpenA, OpenB, LOpenA, LOpenB, ROpenA, ROpenB, ExtA, ExtB, LExtA, LExtB, RExtA, RExtB);
	Log("\n");
	}

void AlnParams::SetMxFromCmdLine(bool IsNucleo)
	{
	//if (IsNucleo)
		SetNucSubstMx(opt_match, opt_mismatch);
	//else
	//	{
	//	if (opt_matrix == "")
	//		{
	//		SubstMxName = "BLOSUM62";
	//		SetBLOSUM62();
	//		}
	//	else
	//		{
	//		ReadSubstMx(opt_matrix, g_SubstMxf);
	//		g_SubstMx = g_SubstMxf.GetData();
	//		g_SubstMxf.LogMe();
	//		SubstMxName = opt_matrix.c_str();
	//		}
	//	}
	SubstMx = g_SubstMx;
	asserta(SubstMx != 0);
	}

void AlnParams::InitFromCmdLine(bool IsNucleo)
	{
	Clear();
	Nucleo = IsNucleo;
	NucleoSet = true;

	SetMxFromCmdLine(IsNucleo);

	Init4(g_SubstMx, -10.0, -1.0, -0.5, -0.5);
	}

float AlnParams::GetLocalOpen() const
	{
	return LocalOpen;
	}

float AlnParams::GetLocalExt() const
	{
	return LocalExt;
	}

bool AlnParams::GetIsNucleo() const
	{
	asserta(NucleoSet);
	return Nucleo;
	}

